unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ComCtrls, ExtCtrls, unitNTHandles, ActnList,
  Menus, ImgList, StdActns, ToolWin, cmpRunOnce,
  cmpStandardSystemMenu, cmpNTAboutBox, GraphUtil, XPMan;

type
  TForm1 = class(TForm)
    ListView1: TListView;
    ActionList1: TActionList;
    actScan: TAction;
    CoolBar1: TCoolBar;
    tbMenu: TToolBar;
    MainMenu1: TMainMenu;
    FileExit1: TFileExit;
    ImageList1: TImageList;
    mnuFile: TMenuItem;
    Exit1: TMenuItem;
    actViewRefresh: TAction;
    View1: TMenuItem;
    Help1: TMenuItem;
    Refresh1: TMenuItem;
    tbMain: TToolBar;
    ToolButton1: TToolButton;
    NTAboutBox1: TNTAboutBox;
    StandardSystemMenu1: TStandardSystemMenu;
    actHelpAbout: TAction;
    About1: TMenuItem;
    FileSaveAs1: TFileSaveAs;
    SaveAs1: TMenuItem;
    N1: TMenuItem;
    ToolButton2: TToolButton;
    XPManifest1: TXPManifest;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ListView1Data(Sender: TObject; Item: TListItem);
    procedure FormShow(Sender: TObject);
    procedure ListView1ColumnClick(Sender: TObject; Column: TListColumn);
    procedure actViewRefreshExecute(Sender: TObject);
    procedure actHelpAboutExecute(Sender: TObject);
    procedure FileSaveAs1Accept(Sender: TObject);
  private
    fHandles : TSystemHandles;
    fFileHandles : TSystemFileHandles;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses unitNTSecurity;

procedure TForm1.FormCreate(Sender: TObject);
begin
  fHandles := TSystemHandles.Create;
  fFileHandles := TSystemFileHandles.Create(fHandles);
  tbMenu.Font := Screen.MenuFont;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  fFileHandles.Free;
  fHandles.Free
end;

procedure TForm1.ListView1Data(Sender: TObject; Item: TListItem);
var
  info : TFileHandleInfo;
begin
  info := fFileHandles.FileHandle [Item.Index];

  Item.Caption := info.FileName;
  Item.SubItems.Add (IntToStr (info.Handle));
  Item.SubItems.Add (IntToStr (info.pid));
  Item.SubItems.Add (info.ProcessName);
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  if isAdmin then
    ListView1.Items.Count := fFileHandles.Count
  else
    ShowMessage ('You must be an administrator to run this program')
end;

procedure TForm1.ListView1ColumnClick(Sender: TObject;
  Column: TListColumn);
begin
  fFileHandles.Sort(TFileHandlesSortType (Column.Index));

  ListView1.Invalidate
end;

procedure TForm1.actViewRefreshExecute(Sender: TObject);
begin
  fHandles.Refresh;

  fFileHandles.Refresh;
  ListView1.Items.Count := fFileHandles.Count;
  ListView1.Refresh
end;

procedure TForm1.actHelpAboutExecute(Sender: TObject);
begin
  NTAboutBox1.Execute
end;

procedure TForm1.FileSaveAs1Accept(Sender: TObject);
var
  f : TFileStream;
  i : Integer;
  info : TFileHandleInfo;
  st : string;

  procedure WriteLn (const st : string);
  begin
    f.Write(st [1], Length (st));
    f.Write (#13#10, 2)
  end;
begin
  f := TFileStream.Create(FileSaveAs1.Dialog.FileName, fmCreate);
  try
    WriteLn ('Process Name          Handle  PID   File Name');

    for i := 0 to fFileHandles.Count - 1 do
    begin
      info := fFileHandles.FileHandle [i];

      st := Format ('%-22s%-8d%-6d%s', [info.ProcessName, info.Handle, info.PID, info.FileName]);
      WriteLn (st)
    end

  finally
    f.Free
  end
end;

end.
